@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-HPI Extended Message Handler Functions
+ HPI Extended Message Handler Functions
 
 (C) Copyright AudioScience Inc. 1997-2003
 ******************************************************************************/
@@ -24,18 +24,25 @@ HPI Extended Message Handler Functions
 #ifndef _HPIMSGX_H_
 #define _HPIMSGX_H_
 
-#include "hpi.h"
+#include "hpi_internal.h"
 
 #ifdef __cplusplus
+/* *INDENT-OFF* */
 extern "C" {
+/* *INDENT-ON* */
 #endif
 
 #define HPIMSGX_ALLADAPTERS     (0xFFFF)
 
-	void HPI_MessageEx(HPI_MESSAGE * phm, HPI_RESPONSE * phr, void *hOwner);
+void hpi_send_recv_ex(struct hpi_message *phm, struct hpi_response *phr,
+	void *h_owner);
+
+#define HPI_MESSAGE_LOWER_LAYER hpi_send_recv_ex
 
 #ifdef __cplusplus
+/* *INDENT-OFF* */
 }
+/* *INDENT-ON* */
 #endif
+
 #endif				/* _HPIMSGX_H_ */
-///////////////////////////////////////////////////////////////////////////
