@@ -1,10 +1,4 @@
-#include "config.h"
-#ifdef CONFIG_HAVE_DEPRECATED_CONFIG_H
-#include <linux/autoconf.h>
-#else
-#include <linux/config.h>
-#endif
-#include <linux/version.h>
+#include "alsa-autoconf.h"
 
 #if LINUX_VERSION_CODE <= KERNEL_VERSION(2,6,15)
 #include "vxpocket_old.c"
